@@ -12,9 +12,10 @@
  * 
  */
 
+#include "include/int_types.h"
+
 #include "common/Thread.h"
 #include "common/admin_socket.h"
-#include "common/admin_socket_client.h"
 #include "common/config.h"
 #include "common/cmdparse.h"
 #include "common/dout.h"
@@ -27,7 +28,6 @@
 
 #include <errno.h>
 #include <fcntl.h>
-#include <inttypes.h>
 #include <map>
 #include <poll.h>
 #include <set>
@@ -184,21 +184,15 @@ std::string AdminSocket::bind_and_listen(const std::string &sock_path, int *fd)
 	   sizeof(struct sockaddr_un)) != 0) {
     int err = errno;
     if (err == EADDRINUSE) {
-      AdminSocketClient client(sock_path);
-      bool ok;
-      client.ping(&ok);
-      if (ok) {
-	ldout(m_cct, 20) << "socket " << sock_path << " is in use" << dendl;
-	err = EEXIST;
-      } else {
-	ldout(m_cct, 20) << "unlink stale file " << sock_path << dendl;
-	TEMP_FAILURE_RETRY(unlink(sock_path.c_str()));
-	if (bind(sock_fd, (struct sockaddr*)&address,
-		 sizeof(struct sockaddr_un)) == 0) {
-	  err = 0;
-	} else {
-	  err = errno;
-	}
+      // The old UNIX domain socket must still be there.
+      // Let's unlink it and try again.
+      TEMP_FAILURE_RETRY(unlink(sock_path.c_str()));
+      if (bind(sock_fd, (struct sockaddr*)&address,
+	       sizeof(struct sockaddr_un)) == 0) {
+	err = 0;
+      }
+      else {
+	err = errno;
       }
     }
     if (err != 0) {
@@ -327,9 +321,6 @@ bool AdminSocket::do_accept()
     return false;
   }
   cmd_getval(m_cct, cmdmap, "format", format);
-  if (format != "json" && format != "json-pretty" &&
-      format != "xml" && format != "xml-pretty")
-    format = "json-pretty";
   cmd_getval(m_cct, cmdmap, "prefix", c);
 
   string firstword;
@@ -453,8 +444,6 @@ public:
   HelpHook(AdminSocket *as) : m_as(as) {}
   bool call(string command, cmdmap_t &cmdmap, string format, bufferlist& out) {
     Formatter *f = new_formatter(format);
-    if (!f)
-      f = new_formatter("json-pretty");
     f->open_object_section("help");
     for (map<string,string>::iterator p = m_as->m_help.begin();
 	 p != m_as->m_help.end();
