@@ -86,6 +86,11 @@ public:
     return ret;
   }
 
+  /// @return true if object is snapdir
+  bool is_snapdir() const {
+    return snap == CEPH_SNAPDIR;
+  }
+
   /// @return snapdir version of this hobject_t
   hobject_t get_snapdir() const {
     hobject_t ret(*this);
@@ -118,6 +123,13 @@ public:
   bool is_max() const {
     return max;
   }
+  bool is_min() const {
+    // this needs to match how it's constructed
+    return snap == 0 &&
+	   hash == 0 &&
+	   !max &&
+	   pool == -1;
+  }
 
   static uint32_t _reverse_nibbles(uint32_t retval) {
     // reverse nibbles
