@@ -143,10 +143,7 @@ void Elector::expire()
     victory();
   } else {
     // whoever i deferred to didn't declare victory quickly enough.
-    if (mon->has_ever_joined)
-      start();
-    else
-      mon->bootstrap();
+    start();
   }
 }
 
